function varargout=max(X,Y,dim)
% -- W = max (X)
% -- W = max (X, [], DIM)
% -- W = max (X, Y)
% -- [W, I] = max (X)
% -- [W, I] = max (X, [], DIM)
% -- [W, I, J] = max (X, [], 3)
%     Find maximum values in amat object X.

%     For the N-by-m-by-n amat X argument, return a N-by-1-by-1 amat .  

%     If the optional third argument DIM is present then operate along
%     particular dimension(s).  In this case the second argument is ignored and
%     should be set to the empty matrix.
%        DIM==0 (default) along N, 
%               return a n-by-m matrix such that (i,j) element is the 
%               maximum of X(:,i,j)
%        DIM==1 along rows of X, 
%               return a N-by-1-by-n amat object such that (k,1,j) element is 
%               the maximum of X(k,:,j)
%        DIM==2 along columns of amat, 
%               return a N-by-m-by-1 amat object such that (k,i,1) element is 
%               the maximum of X(k,i,:)
%        DIM==3 along rows and columns of amat, 
%               return a N-by-1-by-1 amat object such that (k,1,1) element is 
%               the maximum of X(k,:,:) 
%
%     If called with one input and two output arguments, 'max' also returns the 
%     first index of the maximum value(s) as an amat object.
%       [Y,K]=max(X,[],0)
%         =>  Y(i,j)==X(K(i,j),i,j)        ( = maximum of X(:,i,j) )
%       [Y,I]=max(X,[],1)
%         =>  X(k,I(k,1,j),j) == Y(k,1,j)  ( = maximum of X(k,:,j) )
%       [Y,J]=max(X,[],2)
%         =>  X(k,i,J(k,i,1)) == Y(k,i,1)  ( = maximum of X(k,i,1) )
%       [Y,I,J]=max(X,[],3)
%         =>  X(k,I(k,1,1),J(k,1,1)) == Y(k,1,1)  ( = maximum of X(k,:,:) )
  if nargin==1, Y=[];dim=0;end
  if nargin==2, dim=0;end
  if nargin==3
    if ~isempty(Y)
      warning('Input second parameter is forced to []')
      Y=[];
    end
  end
  
  % max (X, [], 3)
  %---------------
  if dim==3 % case W = max (X, [], 3)
    if nargout==1
      varargout{1}=max(max(X,[],2),[],1); 
    else  % case [W, I, J] = max (X, [], 3)
      assert(nargout==3)   
      [Y,I]=max(X,[],1)
      [Z,J]=max(Y,[],2);
      varargout{1}=Z;
      idx=sub2ind(size(I),1:I.N,ones(1,I.N),squeeze(J)');
      varargout{2}=amat(I.values(idx)');
      varargout{3}=J;
    end
    return
  end
  
  assert( isamat(X) )
  assert( ismember(dim,0:2) );
  assert( ismember(nargout,0:2) )
  if dim==0,Amat=@squeeze;else Amat=@amat;end
    
  if isempty(Y) 
    % max (X, [], DIM) with DIM in 0:2
    %---------------
    if nargout==2 
      % case [W , I] = max (X, [], DIM)
      [Y,I]=max(X.values,[],dim+1);
      varargout{1}=Amat(Y);
      varargout{2}=Amat(I);
    else
    % case W = max (X, [], DIM)
      varargout{1}=Amat(max(X.values,[],dim+1));
    end
    return
  end
  
  % W = max (X, Y )
  % ---------------
  getY=fc_amat.getVec(Y,X.N);
  W=amat(X);
  for i=1:X.nr
    for j=1:X.nc
      W.values(:,i,j)=max(X.values(:,i,j),getY(Y,i,j));
    end
  end
  varargout{1}=W;
end
